@@ -48,32 +48,33 @@
 
 #ifdef IPKG_LIB
 #include "libipkg.h"
+static void *p_userdata = NULL;
 #endif
 
-static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_info_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_download_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatdepends_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatdepends_recursively_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatsuggests_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatrecommends_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatprovides_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatconflicts_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_whatreplaces_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_compare_versions_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
-static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char *argv[]);
+static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_info_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_download_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatdepends_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatdepends_recursively_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatsuggests_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatrecommends_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatprovides_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatconflicts_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_whatreplaces_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_compare_versions_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char **argv);
 
 /* XXX: CLEANUP: The usage strings should be incorporated into this
    array for easier maintenance */
@@ -114,7 +115,6 @@ static void write_status_files_if_changed(ipkg_conf_t *conf)
      if (ipkg_state_changed && !conf->noaction) {
 	  ipkg_message(conf, IPKG_INFO,
 		       "  writing status file\n");
-	  fflush(stdout);
 	  ipkg_conf_write_status_files(conf);
 	  pkg_write_changed_filelists(conf);
      }
@@ -138,16 +138,27 @@ ipkg_cmd_t *ipkg_cmd_find(const char *name)
      return NULL;
 }
 
-int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, char *argv[])
+#ifdef IPKG_LIB
+int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, const char **argv, void *userdata)
+{
+	int result;
+	p_userdata = userdata;
+	result = (cmd->fun)(conf, argc, argv);
+	p_userdata = NULL;
+	return result;
+}
+#else
+int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, const char **argv)
 {
      return (cmd->fun)(conf, argc, argv);
 }
+#endif
 
-static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int err;
      int failures;
-     const char *lists_dir;
+     char *lists_dir;
      pkg_src_list_elt_t *iter;
      pkg_src_t *src;
 
@@ -265,7 +276,7 @@ static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 
 
 /* scan the args passed and cache the local filenames of the packages */
-int ipkg_multiple_files_scan(ipkg_conf_t *conf, int argc, char *argv[])
+int ipkg_multiple_files_scan(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      int err;
@@ -337,7 +348,7 @@ static void sigint_handler(int sig)
      exit(128 + sig);
 }
 
-static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      char *arg;
@@ -382,7 +393,7 @@ static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      pkg_t *pkg;
@@ -445,7 +456,7 @@ static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_download_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_download_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i, err;
      char *arg;
@@ -489,7 +500,7 @@ static int ipkg_regcomp(regex_t *re, const char *pattern)
      return err;
 }
 
-static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i, err;
      pkg_vec_t *available;
@@ -526,7 +537,10 @@ static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 #ifndef IPKG_LIB
 	  printf("%s - %s\n", pkg->name, desc_short);
 #else
-	  if (ipkg_cb_list) ipkg_cb_list(pkg->name,desc_short, pkg_version_str_alloc(pkg), pkg->state_status);
+	  if (ipkg_cb_list) ipkg_cb_list(pkg->name,desc_short, 
+		                             pkg_version_str_alloc(pkg), 
+	                                 pkg->state_status, 
+	                                 p_userdata);
 #endif
      }
 
@@ -535,7 +549,7 @@ static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char *argv[], int installed_only)
+static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char **argv, int installed_only)
 {
      int i, j, err;
      pkg_vec_t *available;
@@ -589,17 +603,17 @@ static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char *argv[], int i
      return 0;
 }
 
-static int ipkg_info_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_info_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_info_status_cmd(conf, argc, argv, 0);
 }
 
-static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_info_status_cmd(conf, argc, argv, 1);
 }
 
-static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      char *pkg_name = NULL;
      regex_t re;
@@ -621,7 +635,7 @@ static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return err;
 }
 
-static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i, err;
      char *globpattern;
@@ -659,7 +673,7 @@ static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return err;
 }
 
-static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      pkg_t *pkg;
@@ -730,7 +744,7 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      pkg_t *pkg;
@@ -761,7 +775,7 @@ static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      pkg_t *pkg;
@@ -795,7 +809,7 @@ static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      pkg_t *pkg;
      str_list_t *installed_files;
@@ -827,7 +841,7 @@ static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
 
      if (argc > 0) {
@@ -863,8 +877,8 @@ static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 				   depend_t *possibility = cdepend->possibilities[l];
 				   ipkg_message(conf, IPKG_ERROR, "    %s", possibility->pkg->name);
 				   if (conf->verbosity > 0) {
-					char *ver = pkg_version_str_alloc(possibility->pkg); 
-					ipkg_message(conf, IPKG_NOTICE, " %s", ver);
+					// char *ver = abstract_pkg_version_str_alloc(possibility->pkg); 
+					ipkg_message(conf, IPKG_NOTICE, " %s", possibility->version);
 					if (possibility->version) {
 					     char *typestr = NULL;
 					     switch (possibility->constraint) {
@@ -877,7 +891,7 @@ static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 					     }
 					     ipkg_message(conf, IPKG_NOTICE, " (%s %s)", typestr, possibility->version);
 					}
-					free(ver);
+					// free(ver);
 				   }
 				   ipkg_message(conf, IPKG_ERROR, "\n");
 			      }
@@ -899,7 +913,7 @@ enum what_field_type {
   WHATSUGGESTS
 };
 
-static int ipkg_what_depends_conflicts_cmd(ipkg_conf_t *conf, enum what_field_type what_field_type, int recursive, int argc, char *argv[])
+static int ipkg_what_depends_conflicts_cmd(ipkg_conf_t *conf, enum what_field_type what_field_type, int recursive, int argc, char **argv)
 {
 
      if (argc > 0) {
@@ -1011,31 +1025,31 @@ int pkg_mark_provides(pkg_t *pkg)
      return 0;
 }
 
-static int ipkg_whatdepends_recursively_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatdepends_recursively_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_depends_conflicts_cmd(conf, WHATDEPENDS, 1, argc, argv);
 }
-static int ipkg_whatdepends_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatdepends_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_depends_conflicts_cmd(conf, WHATDEPENDS, 0, argc, argv);
 }
 
-static int ipkg_whatsuggests_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatsuggests_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_depends_conflicts_cmd(conf, WHATSUGGESTS, 0, argc, argv);
 }
 
-static int ipkg_whatrecommends_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatrecommends_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_depends_conflicts_cmd(conf, WHATRECOMMENDS, 0, argc, argv);
 }
 
-static int ipkg_whatconflicts_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatconflicts_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_depends_conflicts_cmd(conf, WHATCONFLICTS, 0, argc, argv);
 }
 
-static int ipkg_what_provides_replaces_cmd(ipkg_conf_t *conf, enum what_field_type what_field_type, int argc, char *argv[])
+static int ipkg_what_provides_replaces_cmd(ipkg_conf_t *conf, enum what_field_type what_field_type, int argc, char **argv)
 {
 
      if (argc > 0) {
@@ -1080,17 +1094,17 @@ static int ipkg_what_provides_replaces_cmd(ipkg_conf_t *conf, enum what_field_ty
      return 0;
 }
 
-static int ipkg_whatprovides_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatprovides_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_provides_replaces_cmd(conf, WHATPROVIDES, argc, argv);
 }
 
-static int ipkg_whatreplaces_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_whatreplaces_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      return ipkg_what_provides_replaces_cmd(conf, WHATREPLACES, argc, argv);
 }
 
-static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i, err;
      regex_t re;
@@ -1124,7 +1138,10 @@ static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 #ifndef IPKG_LIB
 		    printf("%s: %s\n", pkg->name, installed_file);
 #else
-			if (ipkg_cb_list) ipkg_cb_list(pkg->name, installed_file, pkg_version_str_alloc(pkg), pkg->state_status);
+			if (ipkg_cb_list) ipkg_cb_list(pkg->name, 
+						       installed_file, 
+			                               pkg_version_str_alloc(pkg), 
+			                               pkg->state_status, p_userdata);
 #endif			   
 	       }		
 	  }
@@ -1142,7 +1159,7 @@ static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char *argv[])
      return 0;
 }
 
-static int ipkg_compare_versions_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_compare_versions_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      if (argc == 3) {
 	  /* this is a bit gross */
@@ -1164,7 +1181,7 @@ static int ipkg_compare_versions_cmd(ipkg_conf_t *conf, int argc, char *argv[])
 #define HOST_CPU_STR HOST_CPU_STR_(HOST_CPU_FOO)
 #endif
 
-static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char *argv[])
+static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      nv_pair_list_elt_t *l;
 
