@@ -59,7 +59,7 @@ static int postrm_upgrade_old_pkg(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
 static int postrm_upgrade_old_pkg_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 
 static int remove_obsolesced_files(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
-static int install_maintainer_scripts(ipkg_conf_t *conf, pkg_t *pkg);
+static int install_maintainer_scripts(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int remove_disappeared(ipkg_conf_t *conf, pkg_t *pkg);
 static int install_data_files(ipkg_conf_t *conf, pkg_t *pkg);
 static int resolve_conffiles(ipkg_conf_t *conf, pkg_t *pkg);
@@ -139,6 +139,7 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
 	  return IPKG_PKG_HAS_NO_CANDIDATE;
      }
 
+     new->state_flag |= SF_USER;
      if (old) {
 	  old_version = pkg_version_str_alloc(old);
 	  new_version = pkg_version_str_alloc(new);
@@ -401,15 +402,22 @@ static int check_conflicts_for(ipkg_conf_t *conf, pkg_t *pkg)
 {
      int i;
      pkg_vec_t *conflicts = NULL;
-     int level = (conf->force_depends ? IPKG_ERROR : IPKG_NOTICE);
+     int level;
+     const char *prefix;
+     if (conf->force_depends) {
+	  level = IPKG_NOTICE;
+	  prefix = "Warning";
+     } else {
+	  level = IPKG_ERROR;
+	  prefix = "ERROR";
+     }
 
      if (!conf->force_depends)
 	  conflicts = (pkg_vec_t *)pkg_hash_fetch_conflicts(&conf->pkg_hash, pkg);
 
      if (conflicts) {
 	  ipkg_message(conf, level,
-		       "%s: The following packages conflict with %s:\n\t",
-		       conf->force_depends ? "Warning" : "ERROR", pkg->name);
+		       "%s: The following packages conflict with %s:\n\t", prefix, pkg->name);
 	  i = 0;
 	  while (i < conflicts->len)
 	       ipkg_message(conf, level, " %s", conflicts->pkgs[i++]->name);
@@ -422,18 +430,19 @@ static int check_conflicts_for(ipkg_conf_t *conf, pkg_t *pkg)
 
 static int update_file_ownership(ipkg_conf_t *conf, pkg_t *new_pkg, pkg_t *old_pkg)
 {
-     if (old_pkg) {
-	  str_list_t *new_list = pkg_get_installed_files(new_pkg);
-	  str_list_t *old_list = pkg_get_installed_files(old_pkg);
-	  str_list_elt_t *iter;
+     str_list_t *new_list = pkg_get_installed_files(new_pkg);
+     str_list_elt_t *iter;
 
-	  for (iter = new_list->head; iter; iter = iter->next) {
-	       char *new_file = iter->data;
-	       pkg_t *owner = file_hash_get_file_owner(conf, new_file);
-	       if (owner == old_pkg) {
-		    file_hash_set_file_owner(conf, new_file, new_pkg);
-	       }
-	  }
+     for (iter = new_list->head; iter; iter = iter->next) {
+	  char *new_file = iter->data;
+	  pkg_t *owner = file_hash_get_file_owner(conf, new_file);
+	  if (!new_file)
+	       ipkg_message(conf, IPKG_ERROR, "Null new_file for new_pkg=%s\n", new_pkg->name);
+	  if (!owner || (owner == old_pkg))
+	       file_hash_set_file_owner(conf, new_file, new_pkg);
+     }
+     if (old_pkg) {
+	  str_list_t *old_list = pkg_get_installed_files(old_pkg);
 	  for (iter = old_list->head; iter; iter = iter->next) {
 	       char *old_file = iter->data;
 	       pkg_t *owner = file_hash_get_file_owner(conf, old_file);
@@ -481,6 +490,7 @@ static int unpack_pkg_control_files(ipkg_conf_t *conf, pkg_t *pkg)
 {
      int err;
      char *conffiles_file_name;
+     char *root_dir;
      FILE *conffiles_file;
 
      sprintf_alloc(&pkg->tmp_unpack_dir, "%s/%s-XXXXXX", conf->tmp_dir, pkg->name);
@@ -538,7 +548,11 @@ static int unpack_pkg_control_files(ipkg_conf_t *conf, pkg_t *pkg)
 
 	  /* Prepend dest->root_dir to conffile name.
 	     Take pains to avoid multiple slashes. */
-	  sprintf_alloc(&cf_name_in_dest, "%s%s", pkg->dest->root_dir,
+	  root_dir = pkg->dest->root_dir;
+	  if (conf->offline_root)
+	       /* skip the offline_root prefix */
+	       root_dir = pkg->dest->root_dir + strlen(conf->offline_root);
+	  sprintf_alloc(&cf_name_in_dest, "%s%s", root_dir,
 			cf_name[0] == '/' ? (cf_name + 1) : cf_name);
 
 	  /* Can't get an md5sum now, (file isn't extracted yet).
@@ -566,6 +580,8 @@ int pkg_get_installed_replacees(ipkg_conf_t *conf, pkg_t *pkg, pkg_vec_t *instal
 	  if (pkg_vec) {
 	       for (j = 0; j < pkg_vec->len; j++) {
 		    pkg_t *replacee = pkg_vec->pkgs[j];
+		    if (!pkg_conflicts(pkg, replacee))
+			 continue;
 		    if (replacee->state_status == SS_INSTALLED) {
 			 pkg_vec_insert(installed_replacees, replacee);
 		    }
@@ -665,6 +681,7 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
      pkg_t *old_pkg = NULL;
      pkg_vec_t *replacees;
      abstract_pkg_t *ab_pkg = NULL;
+     int old_state_flag;
 
      if (!pkg) {
 	  ipkg_message(conf, IPKG_ERROR,
@@ -702,15 +719,6 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
      err = check_conflicts_for(conf, pkg);
      if (err) { return err; }
     
-     /* We should update the filelist here, so that upgrades of packages that split will not fail. -Jamey 27-MAR-03 */
-     err = update_file_ownership(conf, pkg, old_pkg);
-     if (err) { return err; }
-
-     if (conf->nodeps == 0) {
-	  err = satisfy_dependencies_for(conf, pkg);
-	  if (err) { return err; }
-     }
-
      /* this setup is to remove the upgrade scenario in the end when
 	installing pkg A, A deps B & B deps on A. So both B and A are
 	installed. Then A's installation is started resulting in an
@@ -734,7 +742,15 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	  unpack_pkg_control_files(conf, pkg);
      }
 
-     /* XXX: FEATURE: Need to really support Conflicts/Replaces: here at some point */
+     /* We should update the filelist here, so that upgrades of packages that split will not fail. -Jamey 27-MAR-03 */
+     err = update_file_ownership(conf, pkg, old_pkg);
+     if (err) { return err; }
+
+     if (conf->nodeps == 0) {
+	  err = satisfy_dependencies_for(conf, pkg);
+	  if (err) { return err; }
+     }
+
      replacees = pkg_vec_alloc();
      pkg_get_installed_replacees(conf, pkg, replacees);
 
@@ -753,6 +769,8 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	  }
 
 	  ipkg_state_changed++;
+	  pkg->state_flag |= SF_FILELIST_CHANGED;
+
 	  /* XXX: BUG: we really should treat replacement more like an upgrade
 	   *      Instead, we're going to remove the replacees 
 	   */
@@ -777,6 +795,7 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	  err = postrm_upgrade_old_pkg(conf, pkg, old_pkg);
 	  if (err) goto UNWIND_POSTRM_UPGRADE_OLD_PKG;
 
+	  if (conf->noaction) return 0;
 
 	  /* point of no return: no unwinding after this */
 	  if (old_pkg && !conf->force_reinstall) {
@@ -785,34 +804,31 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	       if (old_pkg->state_flag & SF_NOPRUNE) {
 		    ipkg_message(conf, IPKG_INFO,
 				 "  not removing obsolesced files because package marked noprune\n");
-		    fflush( stdout );
 	       } else {
 		    ipkg_message(conf, IPKG_INFO,
 				 "  removing obsolesced files\n");
-		    fflush( stdout );
 		    remove_obsolesced_files(conf, pkg, old_pkg);
 	       }
 	  }
 	  ipkg_message(conf, IPKG_INFO,
 		       "  installing maintainer scripts\n");
-	  fflush( stdout );
-	  install_maintainer_scripts(conf, pkg);
+	  install_maintainer_scripts(conf, pkg, old_pkg);
 
 	  /* the following just returns 0 */
 	  remove_disappeared(conf, pkg);
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  installing data files\n");
-	  fflush( stdout );
 	  install_data_files(conf, pkg);
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  resolving conf files\n");
-	  fflush( stdout );
 	  resolve_conffiles(conf, pkg);
 
 	  pkg->state_status = SS_UNPACKED;
+	  old_state_flag = pkg->state_flag;
 	  pkg->state_flag &= ~SF_PREFER;
+	  ipkg_message(conf, IPKG_DEBUG, "   pkg=%s old_state_flag=%x state_flag=%x\n", pkg->name, old_state_flag, pkg->state_flag);
 
 	  if (old_pkg && !conf->force_reinstall) {
 	       old_pkg->state_status = SS_NOT_INSTALLED;
@@ -822,7 +838,6 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  cleanup temp files\n");
-	  fflush( stdout );
 	  cleanup_temporary_files(conf, pkg);
 
 	  ab_pkg = pkg->parent;
@@ -831,7 +846,6 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "Done.\n");
-	  fflush( stdout );
 
 	  if (use_signal)
 	       signal(SIGINT, old_handler);
@@ -858,12 +872,10 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  cleanup temp files\n");
-	  fflush( stdout );
 	  cleanup_temporary_files(conf, pkg);
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "Failed.\n");
-	  fflush( stdout );
 	  if (use_signal)
 	       signal(SIGINT, old_handler);
 	  else
@@ -952,7 +964,6 @@ static int preinst_configure(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
      if (err) {
 	  ipkg_message(conf, IPKG_ERROR,
 		       "Aborting installation of %s\n", pkg->name);
-	  fflush(stderr);
 	  return 1;
      }
 
@@ -978,36 +989,44 @@ static int backup_modified_conffiles(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_p
      conffile_list_elt_t *iter;
      conffile_t *cf;
 
+     if (conf->noaction) return 0;
+
      /* Backup all modified conffiles */
      if (old_pkg) {
 	  for (iter = old_pkg->conffiles.head; iter; iter = iter->next) {
+	       char *cf_name;
+	       
 	       cf = iter->data;
+	       cf_name = root_filename_alloc(conf, cf->name);
 
 	       /* Don't worry if the conffile is just plain gone */
-	       if (file_exists(cf->name) && conffile_has_been_modified(conf, cf)) {
-		    err = backup_make_backup(conf, cf->name);
+	       if (file_exists(cf_name) && conffile_has_been_modified(conf, cf)) {
+		    err = backup_make_backup(conf, cf_name);
 		    if (err) {
 			 return err;
 		    }
 	       }
+	       free(cf_name);
 	  }
      }
 
      /* Backup all conffiles that were not conffiles in old_pkg */
      for (iter = pkg->conffiles.head; iter; iter = iter->next) {
+	  char *cf_name;
 	  cf = iter->data;
-
+	  cf_name = root_filename_alloc(conf, cf->name);
 	  /* Ignore if this was a conffile in old_pkg as well */
 	  if (pkg_get_conffile(old_pkg, cf->name)) {
 	       continue;
 	  }
 
-	  if (file_exists(cf->name) && (! backup_exists_for(cf->name))) {
-	       err = backup_make_backup(conf, cf->name);
+	  if (file_exists(cf_name) && (! backup_exists_for(cf_name))) {
+	       err = backup_make_backup(conf, cf_name);
 	       if (err) {
 		    return err;
 	       }
 	  }
+	  free(cf_name);
      }
 
      return 0;
@@ -1047,27 +1066,31 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
      */
      str_list_t *files_list;
      str_list_elt_t *iter;
-     char *file;
 
      int clashes = 0;
 
      files_list = pkg_get_installed_files(pkg);
      for (iter = files_list->head; iter; iter = iter->next) {
-	  file = iter->data;
-	  if (file_exists(file) && (! file_is_dir(file))) {
+	  char *root_filename;
+	  char *filename = iter->data;
+	  root_filename = root_filename_alloc(conf, filename);
+	  if (file_exists(root_filename) && (! file_is_dir(root_filename))) {
 	       pkg_t *owner;
 	       pkg_t *obs;
 	       /* Pre-existing conffiles are OK */
-	       if (backup_exists_for(file)) {
+	       /* @@@@ should have way to check that it is a conffile -Jamey */
+	       if (backup_exists_for(root_filename)) {
 		    continue;
 	       }
 
 	       /* Pre-existing files are OK if force-overwrite was asserted. */ 
 	       if (conf->force_overwrite) {
+		    /* but we need to change who owns this file */
+		    file_hash_set_file_owner(conf, filename, pkg);
 		    continue;
 	       }
 
-	       owner = file_hash_get_file_owner(conf, file);
+	       owner = file_hash_get_file_owner(conf, filename);
 
 	       /* Pre-existing files are OK if owned by the pkg being upgraded. */
 	       if (owner && old_pkg) {
@@ -1076,10 +1099,18 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 		    }
 	       }
 
+	       /* Pre-existing files are OK if owned by a package replaced by new pkg. */
+	       if (owner) {
+		    if (pkg_replaces(pkg, owner)) {
+			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file owned by package %s\n", filename, owner->name);
+			 continue;
+		    }
+	       }
+
 	       /* Pre-existing files are OK if they are obsolete */
-	       obs = hash_table_get(&conf->obs_file_hash, file);
+	       obs = hash_table_get(&conf->obs_file_hash, filename);
 	       if (obs) {
-		    ipkg_message(conf, IPKG_NOTICE, "Pre-exiting file %s is obsolete.  obs_pkg=%s\n", file, obs->name);
+		    ipkg_message(conf, IPKG_INFO, "Pre-exiting file %s is obsolete.  obs_pkg=%s\n", filename, obs->name);
 		    continue;
 	       }
 
@@ -1087,7 +1118,7 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 	       ipkg_message(conf, IPKG_ERROR,
 			    "Package %s wants to install file %s\n"
 			    "\tBut that file is already provided by package ",
-			    pkg->name, file);
+			    pkg->name, filename);
 	       if (owner) {
 		    ipkg_message(conf, IPKG_ERROR,
 				 "%s\n", owner->name);
@@ -1097,6 +1128,7 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 	       }
 	       clashes++;
 	  }
+	  free(root_filename);
      }
      pkg_free_installed_files(pkg);
 
@@ -1166,11 +1198,13 @@ static int remove_obsolesced_files(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
  
 	  /* old file is obsolete */
 	  ipkg_message(conf, IPKG_INFO,
-		       "    removing %s\n", old);
-	  err = unlink(old);
-	  if (err) {
-	       ipkg_message(conf, IPKG_DEBUG, "    Warning: remove %s failed: %s\n", old,
-			    strerror(errno));
+		       "    removing obsolete file %s\n", old);
+	  if (!conf->noaction) {
+	       err = unlink(old);
+	       if (err) {
+		    ipkg_message(conf, IPKG_DEBUG, "    Warning: remove %s failed: %s\n", old,
+				 strerror(errno));
+	       }
 	  }
 
      NOT_OBSOLETE:
@@ -1183,11 +1217,42 @@ static int remove_obsolesced_files(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
      return 0;
 }
 
-static int install_maintainer_scripts(ipkg_conf_t *conf, pkg_t *pkg)
+static int remove_obsolete_maintainer_scripts(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
+{
+     int i;
+     int err = 0;
+     char *globpattern;
+     glob_t globbuf;
+     if (0) {
+	  if (!pkg->dest) {
+	       ipkg_message(conf, IPKG_ERROR, "%s: no dest for package %s\n", __FUNCTION__, pkg->name);
+	       return -1;
+	  }
+	  sprintf_alloc(&globpattern, "%s/%s.*", pkg->dest->info_dir, pkg->name);
+	  err = glob(globpattern, 0, NULL, &globbuf);
+	  free(globpattern);
+	  if (err) {
+	       return err;
+	  }
+	  /* XXXX this should perhaps only remove the ones that are not overwritten in new package.  Jamey 11/11/2003 */
+	  for (i = 0; i < globbuf.gl_pathc; i++) {
+	       ipkg_message(conf, IPKG_DEBUG, "Removing control file %s from old_pkg %s\n",
+			    globbuf.gl_pathv[i], old_pkg->name);
+	       if (!conf->noaction)
+		    unlink(globbuf.gl_pathv[i]);
+	  }
+	  globfree(&globbuf);
+     }
+     return err;
+}
+
+static int install_maintainer_scripts(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
 {
      int ret;
      char *prefix;
 
+     if (old_pkg)
+	  remove_obsolete_maintainer_scripts(conf, pkg, old_pkg);
      sprintf_alloc(&prefix, "%s.", pkg->name);
      ret = pkg_extract_control_files_to_dir_with_prefix(pkg,
 							pkg->dest->info_dir,
@@ -1224,26 +1289,17 @@ static int install_data_files(ipkg_conf_t *conf, pkg_t *pkg)
 	than dpkg. Rather than removing backups at this point, we
 	actually do the data file installation now. See comments in
 	check_data_file_clashes() for more details. */
-     char *list_file_name;
     
      ipkg_message(conf, IPKG_INFO,
 		  "    extracting data files to %s\n", pkg->dest->root_dir);
-     fflush(stdout);
      err = pkg_extract_data_files_to_dir(pkg, pkg->dest->root_dir);
      if (err) {
 	  return err;
      }
 
-     ipkg_message(conf, IPKG_INFO,
-		  "    creating %s.list file\n", pkg->name);
-     fflush(stdout);
-     sprintf_alloc(&list_file_name, "%s/%s.list", pkg->dest->info_dir, pkg->name);
-     err = pkg_extract_data_file_names_to_file(pkg, list_file_name);
-     free(list_file_name);
-
-     if (err) {
+     err = pkg_write_filelist(conf, pkg);
+     if (err)
 	  return err;
-     }
 
      /* XXX: FEATURE: ipkg should identify any files which existed
 	before installation and which were overwritten, (see
@@ -1265,25 +1321,35 @@ static int resolve_conffiles(ipkg_conf_t *conf, pkg_t *pkg)
      conffile_t *cf;
      char *cf_backup;
     
+     if (conf->noaction) return 0;
+
      for (iter = pkg->conffiles.head; iter; iter = iter->next) {
+	  char *root_filename;
 	  cf = iter->data;
+	  root_filename = root_filename_alloc(conf, cf->name);
 
 	  /* Might need to initialize the md5sum for each conffile */
 	  if (cf->value == NULL) {
-	       cf->value = file_md5sum_alloc(cf->name);
+	       cf->value = file_md5sum_alloc(root_filename);
 	  }
 
-	  cf_backup = backup_filename_alloc(cf->name);
+	  if (!file_exists(root_filename)) {
+	       free(root_filename);
+	       continue;
+	  }
+
+	  cf_backup = backup_filename_alloc(root_filename);
 
 	  if (file_exists(cf_backup)) {
 	       if (conf->force_defaults
 		   || user_prefers_old_conffile(cf->name, cf_backup)) {
-		    rename(cf_backup, cf->name);
+		    rename(cf_backup, root_filename);
 	       }
 	       unlink(cf_backup);
 	  }
 
 	  free(cf_backup);
+	  free(root_filename);
      }
 
      return 0;
@@ -1377,7 +1443,9 @@ static int cleanup_temporary_files(ipkg_conf_t *conf, pkg_t *pkg)
 	  }
      }
 
-     if (strncmp(pkg->local_filename, conf->tmp_dir, strlen(conf->tmp_dir)) == 0) {
+     ipkg_message(conf, IPKG_INFO, "cleanup_temporary_files: pkg=%s local_filename=%s tmp_dir=%s\n",
+		  pkg->name, pkg->local_filename, conf->tmp_dir);
+     if (pkg->local_filename && strncmp(pkg->local_filename, conf->tmp_dir, strlen(conf->tmp_dir)) == 0) {
 	  unlink(pkg->local_filename);
 	  free(pkg->local_filename);
 	  pkg->local_filename = NULL;
@@ -1478,7 +1546,7 @@ int ipkg_process_actions_unpack_packages(ipkg_conf_t *conf, pkg_vec_t *pkgs_to_r
      for (i = 0 ; i < pkgs_to_install->len; i++ ) {
 	  pkg_t *pkg = pkgs_to_install->pkgs[i];
 
-	  /* XXX: FEATURE: Need to really support Conflicts/Replaces: here at some point */
+	  /* XXX: FEATURE: Need to really support Provides/Replaces: here at some point */
 	  pkg_vec_t *replacees = pkg_vec_alloc();
 	  pkg_get_installed_replacees(conf, pkg, replacees);
 
@@ -1555,31 +1623,26 @@ int ipkg_process_actions_install(ipkg_conf_t *conf, pkg_vec_t *pkgs_to_install)
 	       if (old_pkg->state_flag & SF_NOPRUNE) {
 		    ipkg_message(conf, IPKG_INFO,
 				 "  not removing obsolesced files because package marked noprune\n");
-		    fflush( stdout );
 	       } else {
 		    ipkg_message(conf, IPKG_INFO,
 				 "  removing obsolesced files\n");
-		    fflush( stdout );
 		    remove_obsolesced_files(conf, pkg, old_pkg);
 	       }
 	  }
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  installing maintainer scripts\n");
-	  fflush( stdout );
-	  install_maintainer_scripts(conf, pkg);
+	  install_maintainer_scripts(conf, pkg, old_pkg);
 
 	  /* the following just returns 0 */
 	  remove_disappeared(conf, pkg);
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  installing data files\n");
-	  fflush( stdout );
 	  install_data_files(conf, pkg);
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  resolving conf files\n");
-	  fflush( stdout );
 	  resolve_conffiles(conf, pkg);
 
 	  pkg->state_status = SS_UNPACKED;
@@ -1592,7 +1655,6 @@ int ipkg_process_actions_install(ipkg_conf_t *conf, pkg_vec_t *pkgs_to_install)
 
 	  ipkg_message(conf, IPKG_INFO,
 		       "  cleanup temp files\n");
-	  fflush( stdout );
 	  cleanup_temporary_files(conf, pkg);
 
 	  if (pkg->parent)
@@ -1684,12 +1746,10 @@ int ipkg_process_actions(ipkg_conf_t *conf, pkg_vec_t *pkgs_to_remove, pkg_vec_t
 
      ipkg_message(conf, IPKG_INFO,
 		  "  cleanup temp files\n");
-     fflush( stdout );
      cleanup_temporary_files(conf, pkg);
 
      ipkg_message(conf, IPKG_INFO,
 		  "Failed.\n");
-     fflush( stdout );
      return err;
 }
 
