@@ -233,10 +233,6 @@ int ipkg_remove_pkg(ipkg_conf_t *conf, pkg_t *pkg)
       * and maintainer scripts left around. So, we make remove like purge, 
       * and take out all the crap :) */
 
-     /* Aman Gupta - remove_data_files_and_list already removes conffiles
-      * which have not been changed. Don't need to call remove_conffiles(),
-      * which deletes all regardless */
-     // remove_conffiles(pkg);
      remove_postrm(conf, pkg);
      pkg->state_status = SS_NOT_INSTALLED;
 
@@ -336,22 +332,6 @@ static int remove_data_files_and_list(ipkg_conf_t *conf, pkg_t *pkg)
      return 0;
 }
 
-static int remove_conffiles(ipkg_conf_t *conf, pkg_t *pkg)
-{
-    conffile_list_elt_t *iter;
-    char *file_name;
-
-    for (iter = pkg->conffiles.head; iter; iter = iter->next) {
-	file_name = iter->data->name;
-
-	ipkg_message(conf, IPKG_INFO, "  deleting %s\n", file_name);
-	if (!conf->noaction)
-	     unlink(file_name);
-    }
-
-    return 0;
-}
-
 static int remove_maintainer_scripts_except_postrm(ipkg_conf_t *conf, pkg_t *pkg)
 {
     int i, err;
